package kd.hrmp.hbjm.business.cascadedata;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.common.util.HRStringUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * @author rd_ningjie
 * @description: 级联数据对比
 * @date 2022/6/6 14:05
 */
public class JobCascadeDataValideApplicatioin extends AbstractCascadeDataValideApplication {

    private static final Log LOG = LogFactory.getLog(JobCascadeDataValideApplicatioin.class);

    private static final String VALIDE_CITY_UNDER_COUNTRY = "valideCityUnderCountry";

    //方案下职位序列校验
    private static final String VALIDE_JOBSEQ_UNDER_SCM = "valideJobSeqUnderScm";

    //方案下职位族校验
    private static final String VALIDE_JOBFAMILY_UNDER_SCM = "valideJobFamilyUnderScm";

    //方案下职位类校验
    private static final String VALIDE_JOBCLASS_UNDER_SCM = "valideJobClassUnderScm";

    private JobCascadeDataValideApplicatioin() {
    }

    private static class CascadeDataValideApplicatioinInstance {
        private static JobCascadeDataValideApplicatioin INSTANCE = new JobCascadeDataValideApplicatioin();
    }

    public static JobCascadeDataValideApplicatioin createInstance() {
        return JobCascadeDataValideApplicatioin.CascadeDataValideApplicatioinInstance.INSTANCE;
    }

    /**
     * @param data
     * @param compareData
     * @param ids
     * @description: 校验职位序列是否在方案下 如果不传id集合，则会查询全表搂数据
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/9 10:36
     */
    public boolean valideJobSeqUnderScm(String data, String compareData, List<Object> ids) {
        try {
            //如果线程变量中没有此service，则初始化一个
            CascadeDataValideService cascadeDataValideService =
                    (CascadeDataValideService) Optional.ofNullable(valideContainer.get().get(VALIDE_JOBSEQ_UNDER_SCM)).orElseGet(() -> {
                        QFilter idFilter = null;
                        idFilter = ObjectUtils.isEmpty(ids) ? idFilter : new QFilter(JobConstans.JOBSEQ_ID, QCP.in, ids);
                        //从db中获取数据
                        CompareEntity entity = CompareEntity.newInstanceFromDb(JobConstans.JOBSCM_ID, JobConstans.JOBSEQ_ID,
                                null, HBJMConstants.PAGE_JOBSEQSCM, new QFilter[]{idFilter});
                        //初始化数据container
                        CascadeDataValideService service = new CascadeDataValideService(entity).initData();
                        valideContainer.get().put(VALIDE_JOBSEQ_UNDER_SCM, service);
                        return service;
                    });
            return cascadeDataValideService.compare(data, compareData);
        } catch (Exception e) {
            LOG.error("JobCascadeDataValideApplicatioin valideJobFamilyUnderSeq", e);
        }
        return false;
    }

    /**
     * @param data
     * @param compareData
     * @param ids
     * @description: 校验职位族是否在方案下 如果不传id集合，则会查询全表搂数据
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/9 10:36
     */
    public boolean valideJobFamilyUnderScm(String data, String compareData, List<Object> ids) {
        try {
            //如果线程变量中没有此service，则初始化一个
            CascadeDataValideService cascadeDataValideService =
                    (CascadeDataValideService) Optional.ofNullable(valideContainer.get().get(VALIDE_JOBFAMILY_UNDER_SCM)).orElseGet(() -> {
                        QFilter idFilter = null;
                        idFilter = ObjectUtils.isEmpty(ids) ? idFilter : new QFilter(JobConstans.JOBFAMILY_ID, QCP.in, ids);
                        //从db中获取数据
                        CompareEntity entity = CompareEntity.newInstanceFromDb(JobConstans.JOBSCM_ID,
                                JobConstans.JOBFAMILY_ID, null, HBJMConstants.PAGE_JOBFAMILYSCM, new QFilter[]{idFilter,});
                        entity.setSelectExtProps(JobConstans.JOBSEQ);
                        //初始化数据container
                        CascadeDataValideService service = new CascadeDataValideService(entity).initData();
                        valideContainer.get().put(VALIDE_JOBFAMILY_UNDER_SCM, service);
                        return service;
                    });
            return cascadeDataValideService.compare(data, compareData);
        } catch (Exception e) {
            LOG.error("JobCascadeDataValideApplicatioin valideJobFamilyUnderScm", e);
        }
        return false;
    }


    /**
     * @param data
     * @param compareData
     * @param ids
     * @description: 校验职位类是否在方案下 如果不传id集合，则会查询全表搂数据
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/9 10:36
     */
    public boolean valideJobClassUnderScm(String data, String compareData, List<Object> ids) {
        try {
            //如果线程变量中没有此service，则初始化一个
            CascadeDataValideService cascadeDataValideService =
                    (CascadeDataValideService) Optional.ofNullable(valideContainer.get().get(VALIDE_JOBCLASS_UNDER_SCM)).orElseGet(() -> {
                        QFilter idFilter = null;
                        idFilter = ObjectUtils.isEmpty(ids) ? idFilter : new QFilter(JobConstans.JOBCLASS_ID, QCP.in, ids);
                        //从db中获取数据
                        CompareEntity entity = CompareEntity.newInstanceFromDb(JobConstans.JOBSCM_ID,
                                JobConstans.JOBCLASS_ID, null, HBJMConstants.PAGE_JOBCLASSSCM, new QFilter[]{idFilter,});
                        //初始化数据container
                        entity.setSelectExtProps(JobConstans.JOBFAMILY);
                        CascadeDataValideService service = new CascadeDataValideService(entity).initData();
                        valideContainer.get().put(VALIDE_JOBCLASS_UNDER_SCM, service);
                        return service;
                    });
            return cascadeDataValideService.compare(data, compareData);
        } catch (Exception e) {
            LOG.error("JobCascadeDataValideApplicatioin valideJobClassUnderScm", e);
        }
        return false;
    }


    /**
     * @param data
     * @param compareData
     * @description: 校验职位族是否在序列下（优先使用上一步方案下查询出的族信息，与对应的序列信息对比）
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/9 10:37
     */
    public boolean valideJobFamilyUnderSeq(String data, String compareData) {
        boolean result = false;
        try {
            //如果对比数据为空，则直接返回true
            if(HRStringUtils.isEmpty(compareData)){
                return true;
            }
            //从上一步的线程变量中获取方案下的职位族，在获取职位族上的序列，与职位序列id对比 ,
            // 如果上一步查询不到数据，或者没有数据，则直接返回true,此处不做存在性校验，由其他校验器校验
            CascadeDataValideService cascadeDataValideService =
                    (CascadeDataValideService) Optional.ofNullable(valideContainer.get().get(VALIDE_JOBFAMILY_UNDER_SCM)).get();
            Optional<CascadeDataValideService> service = Optional.ofNullable(cascadeDataValideService);

            result = valideUnderFromSubEntity(service, JobConstans.JOBSEQ_ID, JobConstans.JOBFAMILY_ID, data, compareData);
        } catch (Exception e) {
            LOG.error("JobCascadeDataValideApplicatioin valideJobFamilyUnderSeq", e);
        }
        return result;
    }

    /**
     * @param data
     * @param compareData
     * @description: 校验职位类是否在族下（优先使用上一步方案下查询出的类信息，与对应的族信息对比）
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/9 10:37
     */
    public boolean valideJobClassUnderFamily(String data, String compareData) {
        boolean result = false;
        try {
            //如果对比数据为空，则直接返回true
            if(HRStringUtils.isEmpty(compareData)){
                return true;
            }
            //从上一步的线程变量中获取方案下的职位类，在获取职位类上的族，与职位族id对比 ,
            // 如果上一步查询不到数据，或者没有数据，则直接返回true,此处不做存在性校验，由其他校验器校验
            CascadeDataValideService cascadeDataValideService =
                    (CascadeDataValideService) Optional.ofNullable(valideContainer.get().get(VALIDE_JOBCLASS_UNDER_SCM)).get();
            Optional<CascadeDataValideService> service = Optional.ofNullable(cascadeDataValideService);
            result = valideUnderFromSubEntity(service, JobConstans.JOBFAMILY_ID, JobConstans.JOBCLASS_ID, data, compareData);
        } catch (Exception e) {
            LOG.error("JobCascadeDataValideApplicatioin valideJobClassUnderFamily", e);
        }
        return result;
    }

    private boolean valideUnderFromSubEntity(Optional<CascadeDataValideService> cascadeDataValideService,
                                             String relactionCol, String compareCol, String data, String compareData) {

        AtomicBoolean result = new AtomicBoolean(false);
        cascadeDataValideService.ifPresent(service -> {
            CompareEntity entity = service.getEntity();
            if (!ObjectUtils.isEmpty(entity) && !ObjectUtils.isEmpty(entity.getDataContainer())) {
                Map<String, DynamicObject> container = entity.getDataContainer();
                //如果是主键值，则直接判断是否存在此key，否则要遍历map进行判断
                if (HRStringUtils.equals(relactionCol, entity.getRelactionCol())
                        && HRStringUtils.equals(compareCol, entity.getCompareCol())) {
                    result.set(container.containsKey(data + entity.CON_SYMBOL + compareData));
                } else {
                    for (Map.Entry<String, DynamicObject> entry : container.entrySet()) {
                        DynamicObject ob = entry.getValue();
                        if (compareData.equals(ob.getString(compareCol)) && data.equals(ob.getString(relactionCol))) {
                            result.set(true);
                            break;
                        }
                    }
                }
            } else {
                //如果没有此container中没有构建数据，则直接返回true，让其他校验器进行数据校验
                result.set(true);
            }
        });
        return result.get();
    }
}