package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.basedata.service.BaseDataServiceImpl;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.util.HRObjectUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;
import kd.hrmp.hbjm.common.util.QFilterHelper;
import org.apache.commons.collections4.CollectionUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author rd_xiangquan_zheng
 * @desc 职位类查询
 * @since 2021/12/9 20:45
 **/
public class JobClassTreeQueryRepository {
    private static class JobClassScmQueryInstance {
        private static JobClassTreeQueryRepository INSTANCE = new JobClassTreeQueryRepository();
    }

    public static JobClassTreeQueryRepository getInstance() {
        return JobClassScmQueryInstance.INSTANCE;
    }

    /**
     * 根据用户查询职位类
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobClass(Long userId, List<Long> orgIds, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
            enableFilter.and("jobseq.enable", QFilter.equals, HRBaseConstants.ENABLED);
            enableFilter.and("jobfamily.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            boolean isRemoveDup = true;
            if (CollectionUtils.isNotEmpty(orgIds)) {
                BaseDataServiceImpl baseDataService = new BaseDataServiceImpl();
                permOrgFilter = baseDataService.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBCLASS, orgIds, isRemoveDup);
            } else {
                DynamicObject[] dynamicObjects = JobClassRepository.getInstance().queryJobClass(userId, isShowDisable);
                if (HRObjectUtils.isEmpty(dynamicObjects)) {
                    return null;
                }
                List<Long> ids = Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
                permOrgFilter = new QFilter("id", QFilter.in, ids);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
                enableFilter.and(new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, "1"));
            } else {
                return null;
            }
        }
        return serviceHelper.query("id,name,enable,jobclasslevel,jobfamily,jobseq,parent",new QFilter[]{enableFilter, QFilterHelper.getInitstatusQFilter()}, orderCondition);
    }

    /**
     * 根据用户查询职位类
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobClass(Long userId, List<Long> orgIds, Boolean isShowDisable, QFilter familyIdQFilter, QFilter scmIdQFilter, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
            enableFilter.and("jobseq.enable", QFilter.equals, HRBaseConstants.ENABLED);
            enableFilter.and("jobfamily.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            boolean isRemoveDup = true;
            if (CollectionUtils.isNotEmpty(orgIds)) {
                BaseDataServiceImpl baseDataService = new BaseDataServiceImpl();
                permOrgFilter = baseDataService.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBCLASS, orgIds, isRemoveDup);
            } else {
                DynamicObject[] dynamicObjects = JobClassRepository.getInstance().queryJobClass(userId, isShowDisable);
                if (HRObjectUtils.isEmpty(dynamicObjects)) {
                    return null;
                }
                List<Long> ids = Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
                permOrgFilter = new QFilter("id", QFilter.in, ids);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
                enableFilter.and(new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, "1"));
            } else {
                return null;
            }
        }
        if (familyIdQFilter != null) {
            enableFilter.and(familyIdQFilter);
        }
        if (scmIdQFilter != null) {
            enableFilter.and(scmIdQFilter);
        }
        return serviceHelper.query("id,name,enable,jobclasslevel,jobfamily,jobseq,parent",new QFilter[]{enableFilter, QFilterHelper.getInitstatusQFilter()}, orderCondition);
    }


    /**
     * 根据用户查询职位族
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobFamily(Long userId, List<Long> orgIds, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
            enableFilter.and("jobseq.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            boolean isRemoveDup = true;
            if (CollectionUtils.isNotEmpty(orgIds)) {
                BaseDataServiceImpl baseDataService = new BaseDataServiceImpl();
                permOrgFilter = baseDataService.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBFAMILIY, orgIds, isRemoveDup);
            } else {
                DynamicObject[] dynamicObjects = JobClassRepository.getInstance().queryJobFamily(userId, isShowDisable);
                if (HRObjectUtils.isEmpty(dynamicObjects)) {
                    return null;
                }
                List<Long> ids = Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
                permOrgFilter = new QFilter("id", QFilter.in, ids);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
                enableFilter.and(new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, "1"));
            } else {
                return null;
            }
        }
        return serviceHelper.query("id,name,enable,jobseq",new QFilter[]{enableFilter, QFilterHelper.getInitstatusQFilter()}, orderCondition);
    }

    /**
     * 根据用户查询职位族
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobFamily(Long userId, List<Long> orgIds, Boolean isShowDisable, QFilter familyIdQFilter, QFilter scmIdQFilter, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
            enableFilter.and("jobseq.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            boolean isRemoveDup = true;
            if (CollectionUtils.isNotEmpty(orgIds)) {
                BaseDataServiceImpl baseDataService = new BaseDataServiceImpl();
                permOrgFilter = baseDataService.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBFAMILIY, orgIds, isRemoveDup);
            } else {
                DynamicObject[] dynamicObjects = JobClassRepository.getInstance().queryJobFamily(userId, isShowDisable);
                if (HRObjectUtils.isEmpty(dynamicObjects)) {
                    return null;
                }
                List<Long> ids = Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
                permOrgFilter = new QFilter("id", QFilter.in, ids);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
                enableFilter.and(new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, "1"));
            } else {
                return null;
            }
        }

        if (familyIdQFilter != null) {
            enableFilter.and(familyIdQFilter);
        }
        if (scmIdQFilter != null) {
            enableFilter.and(scmIdQFilter);
        }
        return serviceHelper.query("id,name,enable,jobseq",new QFilter[]{enableFilter, QFilterHelper.getInitstatusQFilter()}, orderCondition);
    }


    /**
     * 根据用户查询职位序列
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobSeq(Long userId, List<Long> orgIds, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            boolean isRemoveDup = true;
            if (CollectionUtils.isNotEmpty(orgIds)) {
                BaseDataServiceImpl baseDataService = new BaseDataServiceImpl();
                permOrgFilter = baseDataService.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBSEQ, orgIds, isRemoveDup);
            } else {
                DynamicObject[] dynamicObjects = JobSeqRepository.getInstance().queryJobSeqByPerm(userId, isShowDisable);
                if (HRObjectUtils.isEmpty(dynamicObjects)) {
                    return null;
                }
                List<Long> ids = Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
                permOrgFilter = new QFilter("id", QFilter.in, ids);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
                enableFilter.and(new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, "1"));
            } else {
                return null;
            }
        }
        return serviceHelper.queryOriginalArray("id,name,enable",new QFilter[]{enableFilter, QFilterHelper.getInitstatusQFilter()}, orderCondition);
    }

    /**
     * 根据用户查询职位族
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobSeq(Long userId, List<Long> orgIds, Boolean isShowDisable, QFilter seqIdQFilter, QFilter scmIdQFilter, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            boolean isRemoveDup = true;
            if (CollectionUtils.isNotEmpty(orgIds)) {
                BaseDataServiceImpl baseDataService = new BaseDataServiceImpl();
                permOrgFilter = baseDataService.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBSEQ, orgIds, isRemoveDup);
            } else {
                DynamicObject[] dynamicObjects = JobSeqRepository.getInstance().queryJobSeqByPerm(userId, isShowDisable);
                if (HRObjectUtils.isEmpty(dynamicObjects)) {
                    return null;
                }
                List<Long> ids = Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
                permOrgFilter = new QFilter("id", QFilter.in, ids);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
                enableFilter.and(new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, "1"));
            } else {
                return null;
            }
        }
        if (seqIdQFilter != null) {
            enableFilter.and(seqIdQFilter);
        }
        if (scmIdQFilter != null) {
            enableFilter.and(scmIdQFilter);
        }
        return serviceHelper.queryOriginalArray("id,name,enable",new QFilter[]{enableFilter, QFilterHelper.getInitstatusQFilter()}, orderCondition);
    }


}
