package kd.hrmp.hbjm.business.utils;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.ObjectUtils;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang.StringUtils;

import java.util.List;
import java.util.Map;

/**
 * @author rd_ningjie
 * @description:获取内存中的数据（一般使用在上级为其本身，在同一个表格中导入数据或者同一批次初始化等场景）
 * @date 2022/6/21 10:40
 */
public class GetDataFromMemory {

    //数据容器
    private Map<String, DynamicObject> datamap;

    //联合主键连接字符串
    private static final String SEPARATOR = "!!";

    /**
     * @param keywords
     * @param list
     * @description: 初始化数据  以key值作为键值，重新构建map数据
     * @return: void
     * @author rd_ningjie
     * @date: 2022/6/21 11:25
     */
    public void initData(String[] keywords, List<DynamicObject> list) {
        if (ObjectUtils.isEmpty(keywords) || ObjectUtils.isEmpty(list)) {
            return;
        }
        Map<String, DynamicObject> datas = new HashedMap();
        for (DynamicObject dyna : list) {
            datas.put(buildKey(keywords, dyna), dyna);
        }
        datamap = datas;
    }


    /**
     * @param key
     * @description: 判断是否存在
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/21 11:17
     */
    public boolean isExsit(String key) {
        return datamap != null && datamap.containsKey(key);
    }


    /**
     * @param key
     * @description: 通过key获取动态对象
     * @return: kd.bos.dataentity.entity.DynamicObject
     * @author rd_ningjie
     * @date: 2022/6/21 11:16
     */
    public DynamicObject getDynamicObjectByKey(String key) {
        if (!ObjectUtils.isEmpty(datamap)) {
            return datamap.get(key);
        }
        return null;
    }

    private String buildKey(String[] keywords, DynamicObject dynamicObject) {
        if (!ObjectUtils.isEmpty(keywords) && !ObjectUtils.isEmpty(dynamicObject)) {
            String[] result = new String[keywords.length];
            for (int i = 0; i < keywords.length; i++) {
                result[i] = dynamicObject.getString(keywords[i]);
            }
            return StringUtils.join(result, SEPARATOR);
        }
        return "";
    }

}
